SystemOrganization addCategory: #LoomPharo!

LoomProfilerTest subclass: #LoomNonDeterministicTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LoomPharo'!

!LoomNonDeterministicTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/23/2012 15:22'!
testBasic

	self scenario1.
	profiler profile: (Array with: thread1 with: thread2).
	self waitForScenarioEnd.
	profiler stopProfiling.
	
	self assert: profiler profilingTime > 10.
	self assert: profiler numberOfSamplings > 1.
	
	"Thread1 consume twice more the CPU than Thead2. Actually, thread2 is sleeping twice much."

	"self assert: (profiler consumptionTimeOf: thread1) > (profiler consumptionTimeOf: thread2)."
	self assert: (profiler consumptionTimeOf: thread1) >= 0.2.
	self assert: (profiler consumptionTimeOf: thread2) <= 0.7.! !

!LoomNonDeterministicTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 19:38'!
testBasic2

	self scenario2.
	Smalltalk garbageCollectMost.
	
	profiler profile: (Array with: thread1 with: thread2 with: thread3).
	self waitForScenarioEnd.
	profiler stopProfiling.
	
	"Thread1 consume twice more the CPU than Thead2. Actually, thread2 is sleeping twice much."
	self assert: (profiler consumptionTimeOf: thread1) > (profiler consumptionTimeOf: thread2).
	self assert: (profiler consumptionTimeOf: thread1) > (profiler consumptionTimeOf: thread3).
	self assert: (profiler consumptionTimeOf: thread3) >  0.
	self assert: (profiler consumptionTimeOf: thread2) >  0! !

!LoomNonDeterministicTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/23/2012 15:24'!
testBasic2Bis

	Smalltalk garbageCollectMost.
	
	self scenario2.
	profiler profile: (Array with: thread1 with: thread2 with: thread3).
	self waitForScenarioEnd.
	profiler stopProfiling.
	
	"Thread1 consume twice more the CPU than Thead2. Actually, thread2 is sleeping twice much."
	self assert: (profiler consumptionTimeOf: thread1) >= 0.4.
	self assert: (profiler consumptionTimeOf: thread1) < 1.0.

	self assert: (profiler consumptionTimeOf: thread2) >= 0.10.
	self assert: (profiler consumptionTimeOf: thread2) < 0.6.
	
	self assert: (profiler consumptionTimeOf: thread3) > 0.0.
	self assert: (profiler consumptionTimeOf: thread3) < 0.4.
! !

!LoomNonDeterministicTest methodsFor: 'tests' stamp: 'AlexandreBergel 9/12/2012 19:38'!
testDynamicCreationOfThreads

	| nbOriginal |
	nbOriginal := (LoomAllThreadsStrategy forProfiler: profiler) processesToProfile size.
	self runThread1.
	self runThread2.
	
	Smalltalk garbageCollectMost.
	
	profiler profileAllProcesses.
	nbOriginal := profiler numberOfThreads.
	self runThread3.
	self wait2.
	self assert: (profiler numberOfThreads - nbOriginal) = 1.
	
	self waitForScenarioEnd! !

LoomObject subclass: #LoomTime
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LoomPharo'!

!LoomTime class methodsFor: 'time' stamp: 'AlexandreBergel 9/6/2012 09:50'!
currentTimeAsMilliseconds

	^ Time now asMilliSeconds ! !

Object subclass: #LoomExporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LoomPharo'!

!LoomExporter methodsFor: 'exporter' stamp: 'AlexandreBergel 9/6/2012 09:21'!
toVW

	"
	self new toVW
	
	If I'm executed, I generate a st file called Loom-AlexandreBergel.(*version*)-vw5.st in the same folder that contains the image (./contents/resources).
	To import Loom in VW your should have a Loom version already loaded in the VisualWorks image and follow this steps:
	
	1) Remove the Loom package and agree with all the classes removal in the Loom-VWSpecific. 
	2) Create the Loom package again and do right click in the package, select 'file into' and choose the file you exported. 
	3) FInally restore the Loom-VWSpecific package by loading the latest version of it from the public repository.
	
	"
	Gofer new
		squeaksource: 'PackageInfo';
		package: 'PackageInfo-Exporters';
		load.
	
	(Smalltalk at: #VW5PackageExporter)
	        fileOutPackageNamed:  'Loom' 
 	       withNamespace: 'Loom'.
	
	! !

Object subclass: #LoomMonitor
	instanceVariableNames: 'workspace profiler refreshProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LoomPharo'!

!LoomMonitor methodsFor: 'public' stamp: 'AlexandreBergel 12/15/2012 11:56'!
open
	| window |
	Smalltalk garbageCollectMost.
	
	workspace := Workspace new.
	workspace toggleStylingActive.
	window := workspace openLabel: 'Loom'.
	window announcer on: WindowClosed do: [ :ann | profiler stopProfiling. ].
	
	profiler := LoomProfiler new profileAllProcesses.

	self refreshAutomatic! !

!LoomMonitor methodsFor: 'refreshing' stamp: 'AlexandreBergel 9/12/2012 15:17'!
refresh
	| stream |
	stream := WriteStream on: String new.
	self writeHeaderOn: stream.
	profiler 
		threadsDo: [ :loomThread |  loomThread printOn: stream. stream cr ]
		using: LoomConsumptionSortingStrategy new.
	
	workspace setContent: stream contents.! !

!LoomMonitor methodsFor: 'refreshing' stamp: 'AlexandreBergel 9/13/2012 09:34'!
refreshAutomatic

	profiler on: LoomMeasurementReadyEvent do: [ :event | self refresh. "Transcript show: 'refresh from UI'; cr" ]
	
"	refreshProcess := [ [ profiler isProfiling ] whileTrue:  [ 
							self refresh.
							(Delay forMilliseconds: 1000) wait. ] ] forkAt: 43."! !

!LoomMonitor methodsFor: 'reporting' stamp: 'AlexandreBergel 9/5/2012 19:24'!
writeHeaderOn: stream
	stream nextPutAll: 'Loom Inspector, last refresh ', Time now printString, String cr.! !

Object subclass: #LoomRoassal
	instanceVariableNames: 'loom view'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LoomPharo'!

!LoomRoassal methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2012 20:29'!
addElementForThread: aThread

	| element |
	element := ROElement on: aThread.
	element + (ROBox new 
			width: [ :el | el model consumptionTimeInteger ];
			height: [ :el | el model consumptionTimeInteger ]).
			
	element @ (ROPopup text: [ :el | el printString ]).
	view add: element! !

!LoomRoassal methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2012 20:26'!
debug
	self open delete.
	300 timesRepeat: [ view bitmap ]! !

!LoomRoassal methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2012 21:25'!
open

	| t |
	loom := LoomProfiler new.

	t := 0.
	view := ROView new.
	ROPluggableAnimation new
		nbCycles: 10000;
		block: [ t := t + 1.
				(t = 10) ifTrue: [ 
					t := 0.
					self refreshView.
					] ];
		on: view. 

	^ view open.
	! !

!LoomRoassal methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2012 21:22'!
refreshView
	| refreshed threadElements allCurrentThreads elem |
	loom sampleExecution.
	
	refreshed := OrderedCollection new.
	threadElements := self threadElements. 
	allCurrentThreads := loom allCurrentThreads.
	allCurrentThreads do: [ :t | 
		elem := threadElements detect: [ :el | el model == t ]  ifNone: [ nil ].
		elem notNil
			ifTrue: [ elem shapes do: #resetCache ]
			ifFalse: [ self addElementForThread: t ].
			 ].
	
	threadElements do: [ :te | 
		(allCurrentThreads anySatisfy: [ :t | te model == t])
			ifFalse: [ te remove ]
		].
	
	ROHorizontalLineLayout on: self threadElements.
	view signalUpdate ! !

!LoomRoassal methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/14/2012 20:29'!
threadElements
	^ view elements select: [ :e | e model class == LoomThread ]! !
